#import "InteractiveGameSdkPlugin.h"
#import "FlutterPlayerPlugin.h"
#import "FlutterPlayerView.h"
#import "SWGameClient.h"
#import "FlutterMacro.h"

@interface InteractiveGameSdkPlugin ()<FlutterStreamHandler,SWGameClientDelegate>

@property (nonatomic, strong) FlutterMethodChannel* channel;

@property (nonatomic, strong) FlutterEventChannel* eventChannel;

@property (nonatomic, copy) FlutterEventSink eventSink;

@property (nonatomic, strong) FlutterPlayerViewFactory *playerViewFactory;

@property (nonatomic, strong) SWGameClient *gameClient;

@end

@implementation InteractiveGameSdkPlugin

- (void)dealloc {
    NSLog(@"dealloc_____%@",[self class]);
}

+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    
    FlutterMethodChannel* channel = [FlutterMethodChannel
      methodChannelWithName:@"com.ishunwan.interactive/game_client"
            binaryMessenger:[registrar messenger]];
    InteractiveGameSdkPlugin* instance = [[InteractiveGameSdkPlugin alloc] init];
    instance.channel = channel;
    [registrar addMethodCallDelegate:instance channel:channel];
    
    FlutterEventChannel* eventChannel = [FlutterEventChannel eventChannelWithName:@"com.ishunwan.interactive/game_client_event" binaryMessenger:[registrar messenger]];
    instance.eventChannel = eventChannel;
    [eventChannel setStreamHandler:instance];
    
    FlutterPlayerViewFactory* playerViewFactory = [[FlutterPlayerViewFactory alloc] initWithMessenger:registrar.messenger];
    instance.playerViewFactory = playerViewFactory;
    [registrar registerViewFactory:playerViewFactory withId:@"com.ishunwan.interactive/game_player_view"];
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    if ([@"getPlatformVersion" isEqualToString:call.method]) {
      result([@"iOS " stringByAppendingString:[[UIDevice currentDevice] systemVersion]]);
    } else if ([@"init" isEqualToString:call.method]) {
        NSLog(@"调用了初始化init方法 参数：%@",call.arguments);
        if (!self.gameClient) {
            NSString *appId = call.arguments[@"appId"];
            if (!appId) {
                result(@{@"message" : @"gameClient初始化失败 appid 不能为空",@"code" : @"101"});
            }
            NSAssert(appId != nil, @"调用init方法 参数 appid 不能为空");
            self.gameClient = [SWGameClient initGameClientWithAppID:appId delegate:self];
            [self.playerViewFactory setGameClient:self.gameClient];
            result(nil);
        } else {
            result(@{@"message" : @"gameClient已经初始化不需要重复初始化", @"code" : @"101"});
        }
    } else if ([@"debugMode" isEqualToString:call.method]) {
        NSLog(@"调用了debugMode方法 参数：%@",call.arguments);
        BOOL debugMode = [call.arguments[@"debugMode"] boolValue];
        [[NSUserDefaults standardUserDefaults] setBool:debugMode forKey:@"ControlLogEnable"];
        [[NSUserDefaults standardUserDefaults] synchronize];
        [self.gameClient debugMode:debugMode];
        result(nil);
    } else if ([@"joinChannel" isEqualToString:call.method]) {
        NSLog(@"调用了加入频道joinChannel方法 参数：%@",call.arguments);
        if (self.gameClient) {
            NSString *channelId = call.arguments[@"channelId"];
            NSString *uid = call.arguments[@"uid"];
            NSString *token = call.arguments[@"token"];
            NSAssert(channelId != nil, @"加入频道失败 参数 channelId 不能为空");
            NSAssert(uid != nil, @"加入频道失败 参数 uid 不能为空");
            __weak typeof(self) weakSelf = self;
            [self.gameClient joinChannelBychannelId:channelId uid:uid token:token joinSuccess:^(NSString * _Nonnull channel, NSUInteger uid) {
                [weakSelf sendEvent:@"onJoinChannelSuccess" params:@{@"channel" : channelId}];
            } joinFailed:^(NSError * _Nonnull error) {
                [weakSelf sendEvent:@"onJoinChannelFailed" params:@{@"channel" : channelId,@"errorCode" : [NSNumber numberWithInteger:error.code], @"errorMsg" : error.localizedDescription}];
            }];
            result(nil);
        } else {
            result(@{@"message" : @"加入频道失败", @"code" : @"102"});
        }
    } else if ([@"setPlayCloudGame" isEqualToString:call.method]) {
        NSLog(@"调用了setPlayCloudGame方法 参数：%@",call.arguments);
        if (self.gameClient) {
            int gameId = [call.arguments[@"gameId"] intValue];
            int padCount = [call.arguments[@"padCount"] intValue];
            __weak typeof(self) weakSelf = self;
            [self.gameClient setPlayCloudGame:gameId padCount:padCount setSuccess:^{
                [weakSelf sendEvent:@"onSetPlayCloudGameSuccess" params:@{@"gameId" : [NSNumber numberWithInt:gameId], @"padCount" : [NSNumber numberWithInt:padCount]}];
            } setFailed:^(NSError * _Nonnull error) {
                [weakSelf sendEvent:@"onSetPlayCloudGameFailed" params:@{@"gameId" : [NSNumber numberWithInt:gameId], @"padCount" : [NSNumber numberWithInt:padCount],@"errorCode" : [NSNumber numberWithInteger:error.code], @"errorMsg" : error.localizedDescription}];
            }];
            result(nil);
        } else {
            result(@{@"message" : @"设置游戏失败", @"code" : @"104"});
        }
    } else if ([@"leaveChannel" isEqualToString:call.method]) {
        NSLog(@"调用了离开频道leaveChannel方法 参数：%@",call.arguments);
        if (self.gameClient) {
            [self.gameClient leaveChannel];
            result(nil);
        } else {
            result(@{@"message" : @"离开频道失败", @"code" : @"103"});
        }
    } else if ([@"destroy" isEqualToString:call.method]) {
        NSLog(@"调用了销毁destroy方法 参数：%@",call.arguments);
        [self.gameClient destroy];
        [self.playerViewFactory destroy];
        self.gameClient = nil;
        result(nil);
    } else {
      result(FlutterMethodNotImplemented);
    }
}

- (FlutterError* _Nullable)onListenWithArguments:(id _Nullable)arguments eventSink:(FlutterEventSink)eventSink {
    self.eventSink = eventSink;
    return nil;
}
 
- (FlutterError* _Nullable)onCancelWithArguments:(id _Nullable)arguments {
    return nil;
}

- (void)sendEvent:(NSString *)methodName params:(NSDictionary *)params {
    if (self.eventSink) {
        NSMutableDictionary *dict = [NSMutableDictionary new];
        [dict setObject:methodName forKey:@"methodName"];
        if (params) {
            [dict setObject:params forKey:@"args"];
        }
        self.eventSink([dict copy]);
    }
}

#pragma mark -- Delegate
- (void)gameClient:(SWGameClient *)gameClient onErrorNotifyErrorCode:(NSInteger)code errorMsg:(NSString *)errorMsg {
    [self sendEvent:@"onErrorNotify" params:@{@"errorCode" : [NSNumber numberWithInteger:code], @"errorMsg" : errorMsg}];
}

- (void)gameClient:(SWGameClient *)gameClient onUserListUpdate:(NSArray *)userList {
    NSMutableArray *dataArray = [NSMutableArray new];
    for (NSDictionary *dict in userList) {
        [dataArray addObject:dict[@"userId"]];
    }
    [self sendEvent:@"onUserListUpdate" params:@{@"userList" : dataArray.copy}];
}

- (void)gameClient:(SWGameClient *)gameClient onCtrlPadUpdate:(NSArray *)ctrlPadList {
    [self sendEvent:@"onCtrlPadUpdate" params:@{@"ctrlPadList" : ctrlPadList}];
}

- (void)gameClient:(SWGameClient *)gameClient onUserJoin:(NSString *)userId {
    [self sendEvent:@"onUserJoin" params:@{@"userId" : userId?:@""}];
}

- (void)gameClient:(SWGameClient *)gameClient onUserOffline:(NSString *)userId {
    [self sendEvent:@"onUserOffline" params:@{@"userId" : userId?:@""}];
}

- (void)gameClient:(SWGameClient *)gameClient onUserOnline:(NSString *)userId {
    [self sendEvent:@"onUserOnline" params:@{@"userId" : userId?:@""}];
}

- (void)gameClient:(SWGameClient *)gameClient onUserLeave:(NSString *)userId {
    [self sendEvent:@"onUserLeave" params:@{@"userId" : userId?:@""}];
}

- (void)gameClient:(SWGameClient *)gameClient onKick:(NSString *)userId {
    [self sendEvent:@"onKick" params:@{@"userId" : userId?:@""}];
}

- (void)gameClient:(SWGameClient *)gameClient onBan:(NSString *)userId {
    [self sendEvent:@"onBan" params:@{@"userId" : userId?:@""}];
}

- (void)gameClient:(SWGameClient *)gameClient onPlayUpdatePositionList:(NSArray *)padCodeList {
    [self sendEvent:@"onPlayUpdate" params:@{@"positionList" : padCodeList?:@[]}];
}

@end
