//
//  SWPlayer.h
//  PlaySDK
//
//  Created by Dylan on 2018/5/10.
//  Copyright © 2018年 iShunwan. All rights reserved.
//

#import <UIKit/UIKit.h>
//#import "SWLXXHttpManager.h"
//#import "SWHYMessageDelivery.h"
@class SWLXXHttpManager;
@class SWHYMessageDelivery;
@class SWPlayerView;
NS_ASSUME_NONNULL_BEGIN

/** 播放属性*/
typedef NS_ENUM(NSInteger, SWPlayerProperty) {
    SWPlayPropertyQuality,//画质
    SWPlayPropertyFps,//帧率
    SWPlayPropertyBitrate,//码率
    SWPlayPropertyResolution,//分辨率
    SWPlayPropertyEncodeType, //压码方式 1软压 2硬压 0默认值表示软压
    SWPlayPropertyMaxIdr,//最大I帧间隔
};

/** 画质等级 */
typedef NS_ENUM(NSInteger, SWPlayerPlayQualityLevel) {
    SWPlayerPlayQualityDefault = 0,
    SWPlayerPlayQualityHigh = 1,  //高清
    SWPlayerPlayQualityNormal = 2,//普通
    SWPlayerPlayQualityFast = 3,  //高速
    SWPlayerPlayQualityLow = 4,   //极速
    SWPlayerPlayQualityAuto = 5   //自动
};

/** 分辨率等级 */
typedef NS_ENUM(NSInteger, SWPlayerPlayResolutionLevel) {
    SWPlayerPlayResolutionHigh = 1,   //高分辨率
    SWPlayerPlayResolutionMiddle = 2, //中分辨率
    SWPlayerPlayResolutionNormal = 3, //普通分辨率
    SWPlayerPlayResolutionLow = 4,    //低分辨率
};

/** 按键控制指令 */
typedef NS_ENUM(NSInteger, SWPlayerControlCommand) {
    SWPlayerControlCommandBack,    //返回上一页
    SWPlayerControlCommandHome,    //回到桌面
    SWPlayerControlCommandShowTasks//切换多任务
};

/** 悬浮按钮位置 */
typedef NS_ENUM(NSInteger, SWPlayerFloatButtonPosition) {
    SWPlayerFloatButtonPositionLT, //左上
    SWPlayerFloatButtonPositionRT, //右上
    SWPlayerFloatButtonPositionLB, //左下
    SWPlayerFloatButtonPositionRB  //右下
};

@class SWPlayer;

@protocol SWPlayerFloatViewDelegate<NSObject>
@optional
- (void)playQualityChanged:(SWPlayerPlayQualityLevel)playQuality;
- (void)playResolutionChanged:(SWPlayerPlayResolutionLevel)resolutionLevel;
- (void)playBitrateChanged:(int)bitrate;
- (void)playFpsChanged:(int)playFps;
- (void)visibilityChanged:(BOOL)visible;
@end

@protocol SWPlayerDelegate<NSObject>
- (void)playerDidStart:(SWPlayer *)player;
- (void)player:(SWPlayer *)player qualityChanged:(SWPlayerPlayQualityLevel)playQuality;
- (void)player:(SWPlayer *)player resolutionChanged:(SWPlayerPlayResolutionLevel)resolutionLevel;
- (void)player:(SWPlayer *)player bitrateChanged:(int)bitrate;
- (void)player:(SWPlayer *)player fpsChanged:(int)playFps;
- (void)player:(SWPlayer *)player networkDelayChanged:(int)delayInMS;
- (void)player:(SWPlayer *)player failed:(NSError *)error;
- (void)playDidStop:(SWPlayer *)player;
- (void)ScreenOrientationChanged:(SWPlayer *)player portraitChanged:(BOOL)portrait;
@end

@interface SWPlayer : NSObject
@property (nonatomic, weak,   nullable) id<SWPlayerDelegate> delegate;
@property (nonatomic, assign, readonly) BOOL autoChangeQuality;
@property (nonatomic, assign, readonly) BOOL voiceEnable;
@property (nonatomic, assign, readonly) SWPlayerPlayQualityLevel playQuality;



@property(nonatomic,assign) BOOL readIsHidden;
//@property(nonatomic,strong) NSObject *readManager;
//@property (nonatomic,strong) NSObject *readClient;
//@property (nonatomic,strong) NSObject *readDelivery;
//@property (nonatomic,strong) NSString *readChannelID;
//@property (nonatomic,strong) NSDictionary *readPkgDic;

//@property(nonatomic,strong) SWLXXHttpManager *sw_manager;
//@property (nonatomic,strong) SWHYMessageDelivery *delivery;
@property (nonatomic,strong) NSString *channelID;

+ (NSString *)currentVersion;
+ (NSString *)playQualityToString:(SWPlayerPlayQualityLevel)quality;
+ (SWPlayer *)createPlayerWithChannelID:(nonnull NSString *)channelID;
- (void)setupPkgDic:(NSDictionary *)pkgDic;

//control UI
- (void)setupPlayQuality:(SWPlayerPlayQualityLevel)quality;
- (void)setupLoadingView:(nullable UIView *)loadingView landscape:(nullable UIView *)loadingViewLand;
- (void)setupFloatView:(UIView *)floatView landscape:(nullable UIView *)floatViewLand;
- (void)setupFloatButtonWithSize:(CGSize)buttonSize
                           image:(UIImage *)image
                        selImage:(UIImage *)selImage
                    initPosition:(SWPlayerFloatButtonPosition)position;
-(void)setUPFloatButtonIsHidden:(BOOL)IsHidden;
- (void)addPlayView:(SWPlayerView *)playView;
- (void)playWithPackageName:(NSString *)packageName
                  appExtras:(NSString *)appExtras
                     padList:(NSString *)padList;
//control funs
//- (void)startPlay:(NSString *)padCode Failed:(void(^ _Nullable)(NSError * _Nonnull error))FailedBlock;
- (void)stopPlay;


- (BOOL)changePlayQuality:(SWPlayerPlayQualityLevel)playQuality;
- (BOOL)changePlayResolution:(SWPlayerPlayResolutionLevel)playResolution;
- (BOOL)changePlayBitrate:(int)bitrate;
- (void)enableVoice:(BOOL)enable completion:(void(^_Nullable)(BOOL enable))completion;
- (void)sendControlCommand:(SWPlayerControlCommand)command;
- (void)hideFloatViewAnimate:(void (^_Nullable)(BOOL finished))completion;
- (void)showDashboard:(BOOL)show;
- (void)showMessage:(NSString *)message;
@end

NS_ASSUME_NONNULL_END
