import 'interactive_game_player.dart';

typedef EmptyCallback = void Function();

typedef ErrorCallback = void Function(int errorCode);

//加入频道成功回调
typedef OnJoinChannelSuccess = void Function(String channel);

//加入频道失败回调
typedef OnJoinChannelFailed = void Function(
    String channel, int errorCode, String errorMsg);

//设置云游戏成功回调
typedef OnSetPlayCloudGameSuccess = void Function(int gameId, int padCount);

//设置云游戏失败回调
typedef OnSetPlayCloudGameFailed = void Function(
    int gameId, int padCount, int errorCode, String errorMsg);

//用户请求失败时，服务端通知当前用户错误信息
typedef OnErrorNotify = void Function(int errorCode, String errorMsg);

//用户列表更新，进入房间或客户端主动同步后服务端推送所有用户列表通知用户更新用户列表
typedef OnUserListUpdate = void Function(List<dynamic> userList);

//用户设备授权改变时，服务端推送变更内容。仅推送变更部分
typedef OnCtrlPadUpdate = void Function(List<dynamic> ctrlPadList);

//用户加入房间，服务端通知加入用户的全部属性
typedef OnUserJoin = void Function(String userId);

//用户网络异常终端，服务端通知所有用户该用户处于离线状态. 一段时间后(默认15秒), 如用户还没有连接上房间, 则向房间发送 onUserLeave 消息
typedef OnUserOffline = void Function(String userId);

//用户网络中断后在一段时间内重新连接上房间时发送该通知(默认15秒)
typedef OnUserOnline = void Function(String userId);

//用户离开房间，服务端通知返回离开房间用户id
typedef OnUserLeave = void Function(String userId);

//被踢出通知，全体通知
typedef OnKick = void Function(String userId);

//用户被系统封禁，全体通知
typedef OnBan = void Function(String userId);

//房主/管理员设置玩法后通知房间玩法改变
typedef OnPlayUpdate = void Function(List<dynamic> positionList);

class GameClientChannelEventHandler {
  ErrorCallback error;

  OnJoinChannelSuccess onJoinChannelSuccess;

  OnJoinChannelFailed onJoinChannelFailed;

  OnSetPlayCloudGameSuccess onSetPlayCloudGameSuccess;

  OnSetPlayCloudGameFailed onSetPlayCloudGameFailed;

  OnErrorNotify onErrorNotify;

  OnUserListUpdate onUserListUpdate;

  OnCtrlPadUpdate onCtrlPadUpdate;

  OnUserJoin onUserJoin;

  OnUserOffline onUserOffline;

  OnUserOnline onUserOnline;

  OnUserLeave onUserLeave;

  OnKick onKick;

  OnBan onBan;

  OnPlayUpdate onPlayUpdate;

  GameClientChannelEventHandler(
      {this.error,
      this.onJoinChannelSuccess,
      this.onJoinChannelFailed,
      this.onSetPlayCloudGameSuccess,
      this.onSetPlayCloudGameFailed,
      this.onErrorNotify,
      this.onUserListUpdate,
      this.onCtrlPadUpdate,
      this.onUserJoin,
      this.onUserOffline,
      this.onUserOnline,
      this.onUserLeave,
      this.onKick,
      this.onBan,
      this.onPlayUpdate});

  void process(String methodName, Map<String, dynamic> data) {
    switch (methodName) {
      case 'error':
        error?.call(data['errorCode']);
        break;
      case 'onJoinChannelSuccess':
        onJoinChannelSuccess?.call(data['channel']);
        break;
      case 'onJoinChannelFailed':
        onJoinChannelFailed?.call(
            data['channel'], data['errorCode'], data['errorMsg']);
        break;
      case 'onSetPlayCloudGameSuccess':
        onSetPlayCloudGameSuccess?.call(data['gameId'], data['padCount']);
        break;
      case 'onSetPlayCloudGameFailed':
        onSetPlayCloudGameFailed?.call(data['gameId'], data['padCount'],
            data['errorCode'], data['errorMsg']);
        break;
      case 'onErrorNotify':
        onErrorNotify?.call(data['errorCode'], data['errorMsg']);
        break;
      case 'onUserListUpdate':
        onUserListUpdate?.call(data['userList']);
        break;
      case 'onCtrlPadUpdate':
        onCtrlPadUpdate?.call(data['ctrlPadList']);
        break;
      case 'onUserJoin':
        onUserJoin?.call(data['userId']);
        break;
      case 'onUserOffline':
        onUserOffline?.call(data['userId']);
        break;
      case 'onUserOnline':
        onUserOnline?.call(data['userId']);
        break;
      case 'onUserLeave':
        onUserLeave?.call(data['userId']);
        break;
      case 'onKick':
        onKick?.call(data['userId']);
        break;
      case 'onBan':
        onBan?.call(data['userId']);
        break;
      case 'onPlayUpdate':
        onPlayUpdate?.call(data['positionList']);
        break;
      default:
    }
  }
}

typedef OnPlayStarted = void Function(SWGamePlayer gamePlayer, int playTime);

typedef OnPlayNetWorkDelay = void Function(
    SWGamePlayer gamePlayer, int delayInMS);

typedef OnPlayError = void Function(SWGamePlayer gamePlayer, int status,
    int errorFrom, int errorCode, String errorMsg);

typedef OnPlayOrientationChanged = void Function(
    SWGamePlayer gamePlayer, bool portrait);

class GamePlayerChannelEventHandler {
  OnPlayStarted onPlayStarted;

  OnPlayNetWorkDelay onPlayNetWorkDelay;

  OnPlayError onPlayError;

  OnPlayOrientationChanged onPlayOrientationChanged;

  GamePlayerChannelEventHandler(
      {this.onPlayStarted,
      this.onPlayError,
      this.onPlayOrientationChanged,
      this.onPlayNetWorkDelay});

  void process(
      SWGamePlayer gamePlayer, String methodName, Map<String, dynamic> data) {
    switch (methodName) {
      case 'onPlayStarted':
        onPlayStarted?.call(gamePlayer, data['totalTimeSeconds']);
        break;
      case 'onPlayNetWorkDelay':
        onPlayNetWorkDelay?.call(gamePlayer, data['delayInMS']);
        break;
      case 'onPlayError':
        onPlayError?.call(gamePlayer, data['status'], data['errorFrom'],
            data['errorCode'], data['errorMsg']);
        break;
      case 'onPlayOrientationChanged':
        onPlayOrientationChanged?.call(gamePlayer, data['portrait']);
        break;
      default:
    }
  }
}
